// float数据是什么精度的
precision mediump float;

// 采样点的坐标
varying highp vec2 v_texcoord;

// 采样器
uniform sampler2D texture_y;
uniform sampler2D texture_u;
uniform sampler2D texture_v;


uniform highp float distortionFactor;

void main(){
    // get source_uv
    highp vec2 source_uv = v_texcoord;//fragCoord.xy / iResolution.xy;

    // assume your distortion center coordinate is (0.5,0.5),you can use your distortion center instead.
    highp vec2 distortion_center = vec2(0.5, 0.5);

    // Define algorithm dependent variables
    highp float distortion_x;
    highp float distortion_y;
    highp float rr;
    highp float r2;
    highp float theta;

    // define distortion coefficient K1 and K2 ,In most cases we can only adjust K1. then K2 parameters can be adjusted more perfect Effect
    // K1 < 0 is pincushion distortion
    // K1 >=0 is barrel distortion
    highp float distortion_k1 = distortionFactor;
    highp float distortion_k2 = distortionFactor == 0.0 ? 0.0 : 0.5;
    highp vec2 dest_uv;

    //--------------------------Algorithm Start----------------------------------------
    rr = sqrt(pow(source_uv.x - distortion_center.x, 2.0) + pow(source_uv.y - distortion_center.y, 2.0));
    r2 = rr * (1.0 + distortion_k1*pow(rr, 2.0) + distortion_k2*pow(rr, 4.0));
    theta = atan(source_uv.x - distortion_center.x, source_uv.y - distortion_center.y);
    distortion_x = sin(theta) * r2 * 1.0;//1.0 is  scale factor
    distortion_y = cos(theta) * r2 * 1.0;//1.0 is  scale factor
    dest_uv.x = distortion_x + 0.5;
    dest_uv.y = distortion_y + 0.5;
    //--------------------------Algorithm End------------------------------------------


    vec4 c = vec4((texture2D(texture_y, dest_uv).r - 16./255.) * 1.164);
    vec4 U = vec4(texture2D(texture_u, dest_uv).r - 128./255.);
    vec4 V = vec4(texture2D(texture_v, dest_uv).r - 128./255.);
    c += V * vec4(1.596, -0.813, 0, 0);
    c += U * vec4(0, -0.392, 2.017, 0);
    c.a = 1.0;
    gl_FragColor = c;


}

